import checkers.nullness.quals.*;

import java.util.*;

public class NonNullMapValue {

  Map<String,@NonNull String> myMap;

  void testMyMap(String key) {
    @NonNull String value;
    value = myMap.get(key);    // should issue warning
    if (myMap.containsKey(key)) {
      value = myMap.get(key);
    }
    for (String keyInMap : myMap.keySet()) {
      value = myMap.get(key); // should issue warning
    }
    for (String keyInMap : myMap.keySet()) {
        value = myMap.get(keyInMap);
    }
    for (Map.Entry<String,@NonNull String> entry : myMap.entrySet()) {
      String keyInMap = entry.getKey();
      value = entry.getValue();
    }
    for (Iterator<String> iter = myMap.keySet().iterator(); iter.hasNext();) {
      String keyInMap = iter.next();
      //value = myMap.get(keyInMap);
    }
  }

  // Discussion:
  //
  // It can be useful to indicate that all the values in a map are non-null.
  // ("@NonNull" is redundant in this declaration, but I've written it
  // explicitly because it is the annotation we are talking about.)
  //   HashMap<String,@NonNull String> myMap;
  //
  // However, the get method's declaration is misleading (in the context of
  // the nullness type system), since it can always return null no matter
  // whether the map values are non-null:
  //   V get(Object key) { ... return null; }
  //
  // Here are potential solutions:
  //  * Forbid declaring values as non-null.  This is the wrong approach.
  //    (It would also be hard to express syntactically.)
  //  * The checker could recognize a special new annotation on the return
  //    value of get, indicating that its return type isn't merely inferred
  //    from the generic type, but is always nullable.  (This special new
  //    annotations could even be "@Nullable".  A different annotation may
  //    be better, becase in general we would like to issue an error
  //    message when someone applies an annotation to a generic type
  //    parameter.)
  // Additionally, to reduce the number of false positive warnings caused
  // by the fact that get's return value is nullable:
  //  * Build a more specialized sophisticated flow analysis that checks
  //    that the passed key to Map.containsKey() is either checked against
  //    Map.containsKey() or Map.keySet().


}
